(function ($) {
  'use strict';

  Drupal.behaviors.euccx = {
    attach: function (context, settings) {

      $(window).load(function() {
        Drupal.euccx.init();
      });

    }
  };

  Drupal.euccx = {};

  // Entry function for our script.
  // Delegates the jobs to other functions.
  Drupal.euccx.init = function() {
    Drupal.euccx.handleCookieSettingsPage();
    Drupal.euccx.processStatus(Drupal.euccx.getCookieStatus());
  }

  // If we are in the page where the cookie settings' block is included, we need
  // to do some stuff.
  Drupal.euccx.handleCookieSettingsPage = function() {
    var cookieBlock = $('#cookie-tabs');
    // If the cookie management block is not in the page, exit.
    // No need to run extra code for something that is not even in the page
    if (!cookieBlock.length)
      return;

    // Create the jquery ui tabs for the cookie management block
    $('#cookie-tabs').tabs();
    // ... handle the switches
    Drupal.euccx.handleSwitches();
    // ... and attach functionality for the save key
    Drupal.euccx.handleCookieSettingsBehaviors();
  }

  // The function that decides what to do with the current user's status.
  // Responsible for unblocking javascript when needed.
  Drupal.euccx.processStatus = function(status) {
    // If the main eucc cookie is not set, it means that the user has not agreed
    // to anything yet. Remove the offending cookies and move on.
    if (!status['cookie-eucc']) {
      var offendingCookies = Drupal.euccx.getOffendingCookies('');
      Drupal.euccx.removeOffendingCookies(offendingCookies);
      return;
    }

    // If the euccx cookie was never set but the eucc cookie was set (checked
    // for that above) it means that the user agreed to everything so we need to
    // unblock everything and move on.
    if (!status['cookie-euccx']) {
      Drupal.euccx.unblockAll();
      Drupal.euccx.yettUnblockAll();
    }

    // We are now in the case where both cookies are set. We need to surgically
    // unblock code depending on its plugin's settings.
    var pluginNames = Drupal.euccx.getPluginNames();
    var settings = Drupal.settings.euccx;

    for (var i = 0; i < pluginNames.length; i++) {
      // If current plugin is not enabled in the administrative settings we can
      // safely skip to the next plugin.
      if (!Drupal.euccx.isPluginEnabled(pluginNames[i]))
        continue;

      // If current plugin is not in the list of the plugins that the user chose
      // to keep enabled, we move on to the next plugin.
      if ($status['cookie-euccx'].indexOf(pluginNames[i]) === -1)
        continue;


      // If the specific plugin has a YETT blacklist defined, we need to get it
      // and whitelist the set of rules
      var blacklist = (((settings || 
        {}).plugins || 
        {}).pluginNames[i] || 
        {}).blacklist;

      if (blacklist) {
        Drupal.euccx.yettUnblockPlugin(blacklist);
      }

      // If the specific plugin has a js_exclude list, there will be a function 
      // defined that we need to call in order to run the scripts.
      var js_exclude = (((settings || 
        {}).plugins || 
        {}).pluginNames[i] || 
        {}).js_exclude;

      if (js_exclude) {
        Drupal.euccx.unblockPlugin(pluginNames[i]);
      }

    }
  }

  // Make sure that when the user hits the save button, it actually does
  // something.
  Drupal.euccx.handleCookieSettingsBehaviors = function() {
    $('body').on('click', '.cookie-settings-save', function(e) {
      try {
        e.preventDefault();
        // Make sure that the EU Cookie Compliance Module has set its cookie
        Drupal.eu_cookie_compliance.acceptAction();
        var plugin;
        // Create our cookie content
        var cookieData = '';
        $('#cookie-tabs input').each(function() {
          if ($(this).prop('checked')) {
            plugin = $(this).attr("name").match(/-[^-]+$/)[0].substring(1);
            cookieData += plugin + "|";
          }
        });
        // ... and set our cookie
        Drupal.euccx.setCookie(cookieData);
      } catch (e) {
        // Nothing to do here. Something went terribly wrong.
      }
    });
  }

  // Check the cookies that are stored in the user's browser and a return an
  // object with all the relevant information.
  Drupal.euccx.getCookieStatus = function() {
    var status = {};
    var cookieName = (typeof eu_cookie_compliance_cookie_name === 'undefined' ||
      eu_cookie_compliance_cookie_name === '') ? 
      'cookie-agreed' : 
      eu_cookie_compliance_cookie_name;
    var cookieEUCC = Drupal.euccx.getCookie(cookieName);
    var cookieEUCCX = Drupal.euccx.getCookie('cookie-extras-agreed');

    status['cookie-eucc'] = cookieEUCC;
    status['cookie-euccx'] = cookieEUCCX;
    if (typeof cookieEUCCX === 'string') {
      var pluginNames = Drupal.euccx.getPluginNames();
      for (var i = 0; i < pluginNames.length; i++) {
        status[pluginNames[i]] = (cookieEUCCX.indexOf(pluginNames[i]) !== -1);
      }
    }
    return status;
  }

  // Unblocks all plugins that used yett
  Drupal.euccx.yettUnblockAll = function() {
    window.yett.unblock();
  }

  // Whitelist a specific set of scripts that were blocked via yett
  Drupal.euccx.yettUnblockPlugin = function(rules) {
    window.yett.unblock(rules);
  }

  // Unblock all plugins
  Drupal.euccx.unblockAll = function() {
    var plugins = Drupal.euccx.getPluginNames();
    for (var i = 0; i < plugins.length; i++) {
      Drupal.euccx.unblockPlugin(plugins[i]);
    }
  }

  // Given a plugin name execute the function that will unblock its code
  Drupal.euccx.unblockPlugin = function(pluginName) {
    var functionName = 'euccx' + pluginName + 'Load';
    try {
      window[functionName]();
    } catch(e) {}
  }

  // Return an array of the plugin names that are defined
  Drupal.euccx.getPluginNames = function() {
    if (Drupal.settings.euccx && Drupal.settings.euccx.plugins) {
      var pluginNames = [];
      for (var key in Drupal.settings.euccx.plugins) {
        if (Drupal.settings.euccx.plugins.hasOwnProperty(key)) {
          pluginNames.push(key);
        }
      }
      return pluginNames;
    }
    return false;
  }

  // Check whether a plugin is enabled
  Drupal.euccx.isPluginEnabled = function(pluginName) {
    if (Drupal.settings.euccx && Drupal.settings.euccx.plugins &&
      Drupal['settings']['euccx']['plugins'][pluginName] &&
      Drupal['settings']['euccx']['plugins'][pluginName][pluginName] === 1)
      return true;
    return false;
  }

  // Return false when the cookie is not properly set or the cookie value when
  // it is (properly set).
  Drupal.euccx.getCookie = function(cookieName) {
    var cookieValue = $.cookie(cookieName);
    if (typeof(cookieValue) === 'undefined' || (cookieValue === null))
      return false;
    return cookieValue;
  }

  // This is using the exact same code with EU Cookie since our cookie is also
  // respecting most of the settings that the user supplied in EU Cookie
  // Compliance module.
  Drupal.euccx.setCookie = function(content) {
    try {
      var date = new Date();
      var domain = Drupal.settings.eu_cookie_compliance.domain ? 
        Drupal.settings.eu_cookie_compliance.domain : 
        '';
      var path = Drupal.settings.basePath;
      var cookieName = 'cookie-extras-agreed';
      if (path.length > 1) {
        var pathEnd = path.length - 1;
        if (path.lastIndexOf('/') === pathEnd) {
          path = path.substring(0, pathEnd);
        }
      }
      var cookieSession = parseInt(
        Drupal.settings.eu_cookie_compliance.cookie_session
      );

      if (cookieSession) {
        $.cookie(cookieName, content, { path: path, domain: domain });
      } else {
        var lifetime = parseInt(
          Drupal.settings.eu_cookie_compliance.cookie_lifetime
        );
        date.setDate(date.getDate() + lifetime);
        $.cookie(cookieName, content, { 
          expires: date, 
          path: path, 
          domain: domain 
        });
      }

      // Store consent if applicable.
      if (Drupal.settings.eu_cookie_compliance.store_consent) {
        var url = Drupal.settings.basePath;
        url += Drupal.settings.pathPrefix;
        url += 'eu-cookie-compliance/store_consent/banner';

        $.post(url, {}, function (data) { });
      }
    } catch(e) {
    }
  }

  // The switches in the block are always set to off. It's up to us to turn them
  // on, if the cookie stored tells us so.
  Drupal.euccx.handleSwitches = function() {
    // If the cookie is not set, just turn them on by default
    var cookieValue = Drupal.euccx.getCookie('cookie-extras-agreed');
    if (!cookieValue) {
      $('#cookie-tabs input').prop('checked', true);
    } else {
      var plugins = Drupal.settings.euccx.plugins;
      for (var key in plugins) {
        if (plugins.hasOwnProperty(key)) {
          var input = $("input[name*='" + key + "']");
          if (input.length && (cookieValue.indexOf(key) !== -1)) {
            input.prop('checked', true);
          }
        }
      }
    }
  }

  // Check our settings for cookies that plugins declared as being offensive.
  // If the pluginName is empty, return offending cookies from all enabled
  // plugins.
  // Note: this function screens out non-enabled plugins.
  Drupal.euccx.getOffendingCookies = function(pluginName) {
    var offendingCookies = [];
    var settings = Drupal.settings.euccx;
    var pluginNames = Drupal.euccx.getPluginNames();
    for (var i = 0; i < pluginNames.length; i++) {
      if (Drupal.euccx.isPluginEnabled(pluginNames[i])){
        // In order to fully understand the following _black magic_ assignment
        // check this: https://stackoverflow.com/a/4034468/403402
        var cookies_handled = (((settings || 
          {}).plugins || 
          {}).pluginNames[i] || 
          {}).cookies_handled;
        // The below check translates to: if the plugin uses the cookies_handled
        // key AND it's an array AND (either this function was called for all
        // plugins or the plugin that it was called for is the one we're
        // checking right now), add the offending cookies to our array.
        if (cookies_handled && cookies_handled.constructor === Array &&
          (pluginName === '' || pluginName === pluginNames[i])) {
          offendingCookies = offendingCookies.concat(cookies_handled);
        }
      }
    }
    return offendingCookies;
  }

  // We remove the offending cookies by setting their expiration date to the
  // past and their content to the empty string.
  Drupal.euccx.removeOffendingCookies = function(cookies) {
    for (var i = 0; i < cookies.length; i++) {
      var ctd = cookies[i];
      document.cookie = ctd + '=; expires=Thu, 01 Jan 1970 00:00:01 GMT;';
    }
  }
})(jQuery);
